# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.object import Object


class AbstractCellArray(Object):
    r"""
    AbstractCellArray - abstract object to represent cell connectivity
    
    Superclass: Object
    
    AbstractCellArray is an abstract base class for storing a
    connectivity table listing the point ids that make up each cell.
    
    @sa
    CellArray StructuredCellArray
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkAbstractCellArray, obj, update, **traits)
    
    def get_cell_at_id(self, *args):
        """
        get_cell_at_id(self, cellId:int, cellSize:int, cellPoints:(int, ...))
             -> None
        C++: void get_cell_at_id(IdType cellId, IdType &cellSize,
            IdType const *&cellPoints)
        get_cell_at_id(self, cellId:int, cellSize:int, cellPoints:(int, ...),
             ptIds:IdList) -> None
        C++: virtual void get_cell_at_id(IdType cellId,
            IdType &cellSize, IdType const *&cellPoints,
            IdList *ptIds)
        get_cell_at_id(self, cellId:int, pts:IdList) -> None
        C++: virtual void get_cell_at_id(IdType cellId, IdList *pts)
        get_cell_at_id(self, cellId:int, cellSize:int, cellPoints:[int, ...])
             -> None
        C++: virtual void get_cell_at_id(IdType cellId,
            IdType &cellSize, IdType *cellPoints)
        Return the point ids for the cell at cellId.
        
        @warning Subsequent calls to this method may invalidate previous
            call
        results if the internal storage type is not the same as IdType
        and cannot be shared through the cellPoints pointer. In other
        words, the method may not be thread safe. Check if shareable
        (using is_storage_shareable()), or use a CellArrayIterator to
        guarantee thread safety.
        """
        my_args = deref_array(args, [('int', 'int', ('int', Ellipsis)), ('int', 'int', ('int', Ellipsis), 'vtkIdList'), ('int', 'vtkIdList'), ('int', 'int', ['int', Ellipsis])])
        ret = self._wrap_call(self._vtk_obj.GetCellAtId, *my_args)
        return ret

    def get_cell_size(self, *args):
        """
        get_cell_size(self, cellId:int) -> int
        C++: virtual IdType get_cell_size(IdType cellId)
        Return the size of the cell at cellId.
        """
        ret = self._wrap_call(self._vtk_obj.GetCellSize, *args)
        return ret

    def _get_max_cell_size(self):
        return self._vtk_obj.GetMaxCellSize()
    max_cell_size = traits.Property(_get_max_cell_size, desc=\
        r"""
        Returns the size of the largest cell. The size is the number of
        points defining the cell.
        """
    )

    def _get_number_of_cells(self):
        return self._vtk_obj.GetNumberOfCells()
    number_of_cells = traits.Property(_get_number_of_cells, desc=\
        r"""
        Get the number of cells in the array.
        """
    )

    def _get_number_of_connectivity_ids(self):
        return self._vtk_obj.GetNumberOfConnectivityIds()
    number_of_connectivity_ids = traits.Property(_get_number_of_connectivity_ids, desc=\
        r"""
        Get the size of the connectivity array that stores the point ids.
        
        ote Do not confuse this with the deprecated
        get_number_of_connectivity_entries(), which refers to the legacy
        memory layout.
        """
    )

    def _get_number_of_offsets(self):
        return self._vtk_obj.GetNumberOfOffsets()
    number_of_offsets = traits.Property(_get_number_of_offsets, desc=\
        r"""
        Get the number of elements in the offsets array. This will be the
        number of cells + 1.
        """
    )

    def get_offset(self, *args):
        """
        get_offset(self, cellId:int) -> int
        C++: virtual IdType get_offset(IdType cellId)
        Get the offset (into the connectivity) for a specified cell id.
        """
        ret = self._wrap_call(self._vtk_obj.GetOffset, *args)
        return ret

    def deep_copy(self, *args):
        """
        deep_copy(self, ca:AbstractCellArray) -> None
        C++: virtual void deep_copy(AbstractCellArray *ca)
        Perform a deep copy (no reference counting) of the given cell
        array.
        """
        my_args = deref_array(args, [['vtkAbstractCellArray']])
        ret = self._wrap_call(self._vtk_obj.DeepCopy, *my_args)
        return ret

    def initialize(self):
        """
        initialize(self) -> None
        C++: virtual void initialize()
        Free any memory and reset to an empty state.
        """
        ret = self._vtk_obj.Initialize()
        return ret
        

    def is_homogeneous(self):
        """
        is_homogeneous(self) -> int
        C++: virtual IdType is_homogeneous()
        Check if all cells have the same number of vertices.
        
        The return value is coded as:
        * -1 = heterogeneous
        * 0 = Cell array empty
        * n (positive integer) = homogeneous array of cell size n
        """
        ret = self._vtk_obj.IsHomogeneous()
        return ret
        

    def is_storage_shareable(self):
        """
        is_storage_shareable(self) -> bool
        C++: virtual bool is_storage_shareable()
        @return True if the internal storage can be shared as a
        pointer to IdType, i.e., the type and organization of internal
        storage is such that copying of data can be avoided, and instead
        a pointer to IdType can be used.
        """
        ret = self._vtk_obj.IsStorageShareable()
        return ret
        

    def shallow_copy(self, *args):
        """
        shallow_copy(self, ca:AbstractCellArray) -> None
        C++: virtual void shallow_copy(AbstractCellArray *ca)
        Shallow copy ca into this cell array.
        """
        my_args = deref_array(args, [['vtkAbstractCellArray']])
        ret = self._wrap_call(self._vtk_obj.ShallowCopy, *my_args)
        return ret

    _updateable_traits_ = \
    (('debug', 'GetDebug'), ('global_warning_display',
    'GetGlobalWarningDisplay'), ('object_name', 'GetObjectName'),
    ('reference_count', 'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['debug', 'global_warning_display', 'object_name'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(AbstractCellArray, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit AbstractCellArray properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View(([], [], ['object_name']),
            title='Edit AbstractCellArray properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit AbstractCellArray properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

