# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.object import Object


class CellGridResponders(Object):
    r"""
    CellGridResponders - A container that holds objects able to
    respond to queries
             specialized for particular CellMetadata types.
    
    Superclass: Object
    
    This class holds sets of responders for CellGridQuery and for
    CellAttributeQuery.
    
    ## Cell-grid query responders
    
    CellGridResponders holds a list of objects statically registered
    to the CellMetadata subclass. These objects respond to queries for
    information (e.g., a bounding box) or processing (e.g., rendering,
    picking, generating isocontours) for one cell type (and subclasses of
    that cell type if no more specific responder is registered).
    
    Application code (such as a plugin) can register subclasses of
    CellGridResponse which accept the API of a particular
    CellGridQuery for that cell type. Then, when a query is passed to
    the cell, this collection will identify matching responders for the
    query and invoke them until one returns true (indicating success).
    Multiple matches can exist as a responder can be registered to a
    common base cell class and/or to handle common base query classes.
    
    If a given cell type cannot respond to a query, its superclasses are
    asked to respond. If no superclass can respond to the query, then
    query's superclasses are searched for responders.
    
    Because queries can be registered to cell types at any time, existing
    cell types can be extended to support new features by additional
    libraries.
    
    ## Cell-attribute calculators
    
    In order to support the evaluation of CellAttribute data on any
    CellMetadata (cell type), this class also holds "calculators"
    grouped by both attribute and cell type. This API is different that
    CellGridQuery because CellAttribute is not subclassed by
    attribute type but rather uses StringToken data to determine the
    nature of the attribute (e.g., Lagrange; Nedelec; Raviart-Thomas;
    etc.).
    
    It is also different in that these objects are expected to be used
    inside CellGridQuery to evaluate a single cell at a time rather
    than longer-running queries across all cells.
    
    @sa CellMetadata CellGrid CellAttribute
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkCellGridResponders, obj, update, **traits)
    
    def _get_cache_data(self):
        return wrap_vtk(self._vtk_obj.GetCacheData())
    def _set_cache_data(self, arg):
        old_val = self._get_cache_data()
        self._wrap_call(self._vtk_obj.SetCacheData,
                        arg)
        self.trait_property_changed('cache_data', old_val, arg)
    cache_data = traits.Property(_get_cache_data, _set_cache_data, desc=\
        r"""
        Return a cache object given a key.
        
        Objects held in this cache may live for the duration of an
        application, not a single query.
        
        Cached objects are expected to invalidate themselves (say by
        using Command to observe events that should invalidate them).
        """
    )

    def query(self, *args):
        """
        query(self, cellType:CellMetadata, query:CellGridQuery)
            -> bool
        C++: bool query(CellMetadata *cellType,
            CellGridQuery *query)
        Invoke a responder for the given query and cell type.
        
        If none exists, return false.
        """
        my_args = [deref_vtk(x) for x in args]
        ret = self._wrap_call(self._vtk_obj.Query, *my_args)
        return ret

    _updateable_traits_ = \
    (('debug', 'GetDebug'), ('global_warning_display',
    'GetGlobalWarningDisplay'), ('object_name', 'GetObjectName'),
    ('reference_count', 'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['debug', 'global_warning_display', 'object_name'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(CellGridResponders, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit CellGridResponders properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View(([], [], ['object_name']),
            title='Edit CellGridResponders properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit CellGridResponders properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

