# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.cell_grid_query import CellGridQuery


class CellGridSidesQuery(CellGridQuery):
    r"""
    CellGridSidesQuery - A cell-grid query for enumerating sides of
    cells.
    
    Superclass: CellGridQuery
    
    This query runs in 3 passes (see CellGridSidesQuery::PassWork):
    
    + In the first pass, responders invoke the add_sides() method on
      this query, entries are added to this->Hashes storage indicating
      the cells which are bounded by a given shape + connectivity. + In
    the second pass, responders mark the entries created above and
      add entries in this->Sides. This reorganizes the hashes into groups
      more amenable to output as side arrays. This pass is called
      "Summarization," since not every input side identified will be
      output. + In the third and final pass, responders create new cells
    in
      the output cell-grid that correspond to the selected sides of
      the input.
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkCellGridSidesQuery, obj, update, **traits)
    
    omit_sides_for_renderable_inputs = tvtk_base.false_bool_trait(desc=\
        r"""
        Set/get whether to omit computation of sides for renderable
        cells.
        
        A cell is renderable if it is of dimension 2 or less (i.e.,
        surfaces, edges, and vertices are all renderable; volumetric
        cells are not). This setting, when used in combination with
        preserve_renderable_inputs, allows the filter to behave similar to
        PolyData surface extraction filters; volumetric cells will
        have sides computed but others will be passed through from the
        input unaltered.
        
        The default is false.
        """
    )

    def _omit_sides_for_renderable_inputs_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetOmitSidesForRenderableInputs,
                        self.omit_sides_for_renderable_inputs_)

    output_dimension_control = tvtk_base.true_bool_trait(desc=\
        r"""
        Set/get which sides to generate given input cells/sides.
        
        output_dimension_control is a bit-vector taking values from the
        side_flags enumeration. It determines which sides of the input
        should be generated. The default is side_flags::surfaces_of_inputs,
        which will only emit surfaces of volumetric cells.
        """
    )

    def _output_dimension_control_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetOutputDimensionControl,
                        self.output_dimension_control_)

    preserve_renderable_inputs = tvtk_base.false_bool_trait(desc=\
        r"""
        Set/get whether renderable cells should be included in the output
        or the output should strictly contain sides of cells.
        
        A cell is renderable if it is of dimension 2 or less (i.e.,
        surfaces, edges, and vertices are all renderable; volumetric
        cells are not).
        
        The default is false.
        """
    )

    def _preserve_renderable_inputs_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetPreserveRenderableInputs,
                        self.preserve_renderable_inputs_)

    strategy = tvtk_base.RevPrefixMap({'boundary': 2, 'any_occurrence': 1, 'winding': 0}, default_value='boundary', desc=\
        r"""
        Set/get the strategy responders should use to generate entries in
        Sides from entries in side_cache.
        
        The default is boundary_strategy.
        """
    )

    def _strategy_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetStrategy,
                        self.strategy_)

    def _get_side_cache(self):
        return wrap_vtk(self._vtk_obj.GetSideCache())
    def _set_side_cache(self, arg):
        old_val = self._get_side_cache()
        self._wrap_call(self._vtk_obj.SetSideCache,
                        deref_vtk(arg))
        self.trait_property_changed('side_cache', old_val, arg)
    side_cache = traits.Property(_get_side_cache, _set_side_cache, desc=\
        r"""
        Set/get cached hashtable of sides.
        
        The idea is that CellGridSidesCache is generic enough to
        accommodate a wide variety of cell types and that many of them
        will be capable of having sides that are conformal to cells of
        different types that may reside in the same cell-grid. Filters
        may own this cache or they may attach it to a collection of
        cell-grid objects that participate by inserting their cells'
        sides into the cache. (For example, all the cell-grids within a
        partitioned dataset collection may wish to insert sides in the
        same cache.)
        """
    )

    def selection_mode_from_label(self, *args):
        """
        selection_mode_from_label(token:StringToken) -> SelectionMode
        C++: static SelectionMode selection_mode_from_label(
            StringToken token)"""
        my_args = [deref_vtk(x) for x in args]
        ret = self._wrap_call(self._vtk_obj.SelectionModeFromLabel, *my_args)
        return ret

    def selection_mode_to_label(self, *args):
        """
        selection_mode_to_label(mode:SelectionMode) -> StringToken
        C++: static StringToken selection_mode_to_label(
            SelectionMode mode)
        Return a string-token with the given selection mode or
        vice-versa.
        """
        ret = self._wrap_call(self._vtk_obj.SelectionModeToLabel, *args)
        return wrap_vtk(ret)

    def summary_strategy_from_label(self, *args):
        """
        summary_strategy_from_label(token:StringToken) -> SummaryStrategy
        C++: static SummaryStrategy summary_strategy_from_label(
            StringToken token)"""
        my_args = [deref_vtk(x) for x in args]
        ret = self._wrap_call(self._vtk_obj.SummaryStrategyFromLabel, *my_args)
        return ret

    def summary_strategy_to_label(self, *args):
        """
        summary_strategy_to_label(strategy:SummaryStrategy) -> StringToken
        C++: static StringToken summary_strategy_to_label(
            SummaryStrategy strategy)
        Return a string-token with the given summarization strategy or
        vice-versa.
        """
        ret = self._wrap_call(self._vtk_obj.SummaryStrategyToLabel, *args)
        return wrap_vtk(ret)

    _updateable_traits_ = \
    (('omit_sides_for_renderable_inputs',
    'GetOmitSidesForRenderableInputs'), ('preserve_renderable_inputs',
    'GetPreserveRenderableInputs'), ('debug', 'GetDebug'),
    ('global_warning_display', 'GetGlobalWarningDisplay'), ('strategy',
    'GetStrategy'), ('object_name', 'GetObjectName'), ('reference_count',
    'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ('output_dimension_control',)
    
    _full_traitnames_list_ = \
    (['debug', 'global_warning_display',
    'omit_sides_for_renderable_inputs', 'preserve_renderable_inputs',
    'strategy', 'object_name'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(CellGridSidesQuery, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit CellGridSidesQuery properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View((['omit_sides_for_renderable_inputs',
            'preserve_renderable_inputs'], ['strategy'], ['object_name']),
            title='Edit CellGridSidesQuery properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit CellGridSidesQuery properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

