# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.pass_input_type_algorithm import PassInputTypeAlgorithm


class CriticalTime(PassInputTypeAlgorithm):
    r"""
    CriticalTime - Compute time step at which a threshold value has
    been reached
    
    Superclass: PassInputTypeAlgorithm
    
    Given an input that changes over time, the `vtkcritical_time` filter
    generates an output with similar structure, with a new data array
    containing time step values (other arrays are discarded). These
    values correspond to the time at which a specified threshold
    criterion has been met for a given point/cell array (at each
    point/cell respectively). To do so, this filter processes all
    available time steps. If the criterion is never met for a given
    point/cell, a NaN value is assigned. The output of this filter is not
    temporal.
    
    The threshold criterion can take three forms:
    1) greater than a particular value;
    2) less than a particular value;
    3) between two values.
    
    When the selected array has more than one component, you can use the
    `set_component_mode` and `set_selected_component` methods to control
    which component(s) are considered to check against the threshold
    criterion:
    1) if the component mode is set to `use_selected`, the `Selected
       Component` is used. The magnitude can be selected if
       selected_component is set to the number of components of the array;
    2) if the component mode is set to `use_any`, only one component needs
    to meet the criterion;
    3) if the component mode is set to `use_all`, all component needs to
       meet the criterion.
    
    The output of this filter correspond to the input with the extra
    temporal field attached to the points/cells. The name of this array
    is the same of the selected array with `_critical_time` appended at
    the end.
    
    ote This filters expects that the input topology do not change over
    time.
    
    ote If the key
    `vtkstreaming_demand_driven_pipeline::NO_PRIOR_TEMPORAL_ACCESS()` is
    set, typically when running this filter in situ, then the filter runs
    the time steps one at a time. It requires causing the execution of
    the filter multiple times externally, by calling `update_time_step()`
    in a loop or using another filter that iterates over time downstream,
    for example. When the key is not set, the filter will execute itself
    by setting the key
    `vtkstreaming_demand_driven_pipeline::CONTINUE_EXECUTING()`. In such
    case, this filter will produce an array called `time_steps` in the
    output's `field_data`. It contains all the time steps that have been
    processed so far.
    
    @sa TemporalAlgorithm TemporalStatistics Threshold
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkCriticalTime, obj, update, **traits)
    
    component_mode = tvtk_base.RevPrefixMap({'use_selected': 0, 'use_all': 1, 'use_any': 2}, default_value='use_selected', desc=\
        r"""
        Control how the decision of in / out is made with multi-component
        data. The choices are to use the selected component (specified in
        the selected_component ivar), or to look at all components. When
        looking at all components, the evaluation can pass if all the
        components satisfy the rule (use_all) or if any satisfy is
        (use_any). The default value is use_selected.
        """
    )

    def _component_mode_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetComponentMode,
                        self.component_mode_)

    threshold_criterion = tvtk_base.RevPrefixMap({'upper': 0, 'lower': 0, 'between': 0}, default_value='upper', desc=\
        r"""
        Get/Set the threshold criterion, defining which threshold bounds
        to use. The default method is `THRESHOLD_UPPER`.
        
        Note: values are clamped between THRESHOLD_BETWEEN and
        THRESHOLD_UPPER.
        """
    )

    def _threshold_criterion_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetThresholdCriterion,
                        self.threshold_criterion_)

    lower_threshold = traits.Float(-inf, enter_set=True, auto_set=False, desc=\
        r"""
        
        """
    )

    def _lower_threshold_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetLowerThreshold,
                        self.lower_threshold)

    selected_component = traits.Trait(0, traits.Range(0, 2147483647, enter_set=True, auto_set=False), desc=\
        r"""
        When the component mode is use_selected, this ivar indicated the
        selected component. If set to the number of components of the
        array, threshold will apply on array's magnitude. The default
        value is 0.
        """
    )

    def _selected_component_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetSelectedComponent,
                        self.selected_component)

    upper_threshold = traits.Float(inf, enter_set=True, auto_set=False, desc=\
        r"""
        
        """
    )

    def _upper_threshold_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetUpperThreshold,
                        self.upper_threshold)

    def _get_input(self):
        return wrap_vtk(self._vtk_obj.GetInput())
    input = traits.Property(_get_input, desc=\
        r"""
        Get the input data object. This method is not recommended for
        use, but lots of old style filters use it.
        """
    )

    def _get_threshold_function_as_string(self):
        return self._vtk_obj.GetThresholdFunctionAsString()
    threshold_function_as_string = traits.Property(_get_threshold_function_as_string, desc=\
        r"""
        Return a string representation of the threshold criterion
        """
    )

    def time_steps_array_name(self):
        """
        time_steps_array_name() -> str
        C++: static const char *time_steps_array_name()"""
        ret = self._vtk_obj.TimeStepsArrayName()
        return ret
        

    _updateable_traits_ = \
    (('abort_execute', 'GetAbortExecute'), ('release_data_flag',
    'GetReleaseDataFlag'), ('debug', 'GetDebug'),
    ('global_warning_display', 'GetGlobalWarningDisplay'),
    ('component_mode', 'GetComponentMode'), ('threshold_criterion',
    'GetThresholdCriterion'), ('lower_threshold', 'GetLowerThreshold'),
    ('selected_component', 'GetSelectedComponent'), ('upper_threshold',
    'GetUpperThreshold'), ('abort_output', 'GetAbortOutput'),
    ('progress_text', 'GetProgressText'), ('object_name',
    'GetObjectName'), ('reference_count', 'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['abort_execute', 'debug', 'global_warning_display',
    'release_data_flag', 'component_mode', 'threshold_criterion',
    'abort_output', 'lower_threshold', 'object_name', 'progress_text',
    'selected_component', 'upper_threshold'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(CriticalTime, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit CriticalTime properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View(([], ['component_mode', 'threshold_criterion'], ['abort_output',
            'lower_threshold', 'object_name', 'selected_component',
            'upper_threshold']),
            title='Edit CriticalTime properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit CriticalTime properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

