# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.object import Object


class DataObjectMeshCache(Object):
    r"""
    DataObjectMeshCache - DataObjectMeshCache is a class to store
    and reuse the mesh of a DataSet, while forwarding data arrays from
    another dataset.
    
    Superclass: Object
    
    Composite structure of DataSet are also supported. This is
    specially useful when working with static meshes and transient data.
    
    ## Example of use case Think about the GeometryFilter. On first
       execution, it extracts boudaries of an input mesh alongside with
       the associated data. On the second execution, if the input mesh
       didn't change and neither did the GeometryFitlers own
       properties, then there is no need for boundary extraction:
       previous output mesh can be reused. Only the associated data
       should be forwarded.
    
    Instead of implementing such logic itself, filtre like the
    GeometryFilter can instead rely on the DataObjectMeshCache in
    order to easily reuse the previously computed mesh, and forward the
    new data arrays.
    
    ## Details This helper relies on different elements:
    - `Consumer`: a Object using the helper. Any modifications on it
      invalidate cache.
    - `original_data_object`: the input DataObject. Should be either a
      DataSet or a composite of DataSet. The helper looks for its
    mesh_m_time.
    - `Cache`: the output DataObject containing the mesh to reuse
      (or a composite)
    - `original_ids`: a list of original ids array name per attribute
      types, to forward from original_data_object to Cache when asked to.
    
    The `Status` structure reflects the state of those different
    elements. It is the user responsibility to check for the status
    before calling `copy_cache_to_data_object`.
    
    Attributes data are forwarded with `copy_allocate` method. So output
    should be a subset of the input. Support for `interpolate_allocate` is
    doable and may be added in the future.
    
    ## Requirements The data arrays forwarding rely on global_ids
        arrays.
    
    When using CompositeDataSet, every leaves should be of a supported
    data set type.
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkDataObjectMeshCache, obj, update, **traits)
    
    def add_original_ids(self, *args):
        """
        add_original_ids(self, attribute:int, name:str) -> None
        C++: void add_original_ids(int attribute, const std::string &name)
        Add original ids array name for attribute type.
        @sa remove_original_ids, clear_original_ids, copy_cache_to_output
        """
        ret = self._wrap_call(self._vtk_obj.AddOriginalIds, *args)
        return ret

    def clear_original_ids(self):
        """
        clear_original_ids(self) -> None
        C++: void clear_original_ids()
        Clear all original ids.
        @sa remove_original_ids, add_original_ids
        """
        ret = self._vtk_obj.ClearOriginalIds()
        return ret
        

    def copy_cache_to_data_object(self, *args):
        """
        copy_cache_to_data_object(self, output:DataObject) -> None
        C++: void copy_cache_to_data_object(DataObject *output)
        Fill given dataset with cached data. If original ids are present,
        copy corresponding attributes. It is the user responsiblity to
        check the status before calling this.
        """
        my_args = [deref_vtk(x) for x in args]
        ret = self._wrap_call(self._vtk_obj.CopyCacheToDataObject, *my_args)
        return ret

    def invalidate_cache(self):
        """
        invalidate_cache(self) -> None
        C++: void invalidate_cache()
        Invalidate cache. Remove cached dataset and reset cached MTimes.
        """
        ret = self._vtk_obj.InvalidateCache()
        return ret
        

    def is_supported_data(self, *args):
        """
        is_supported_data(self, dataobject:DataObject) -> bool
        C++: bool is_supported_data(DataObject *dataobject)
        Return true if dataobject is of a supported type.
        """
        my_args = [deref_vtk(x) for x in args]
        ret = self._wrap_call(self._vtk_obj.IsSupportedData, *my_args)
        return ret

    def remove_original_ids(self, *args):
        """
        remove_original_ids(self, attribute:int) -> None
        C++: void remove_original_ids(int attribute)
        Remove ids array name for attribute type.
        @sa add_original_ids, clear_original_ids, copy_cache_to_output
        """
        ret = self._wrap_call(self._vtk_obj.RemoveOriginalIds, *args)
        return ret

    def set_consumer(self, *args):
        """
        set_consumer(self, _arg:Algorithm) -> None
        C++: virtual void set_consumer(Algorithm *_arg)
        Set the consumer of this cache. The status is invalid if the
        Consumer is modified after the last copy_cache_to_data_object call.
        Required before any call to copy_cache_to_data_object.
        """
        my_args = [deref_vtk(x) for x in args]
        ret = self._wrap_call(self._vtk_obj.SetConsumer, *my_args)
        return ret

    def set_original_data_object(self, *args):
        """
        set_original_data_object(self, original:DataObject) -> None
        C++: void set_original_data_object(DataObject *original)
        Set the original dataobject. The status becomes invalid if the
        original dataobject mesh is modified. Original dataobject is also
        used to copy data arrays to output, if original_ids are
        configured. Required before any call to copy_cache_to_data_object.
        @sa add_original_ids, remove_original_ids, clear_original_ids
        """
        my_args = [deref_vtk(x) for x in args]
        ret = self._wrap_call(self._vtk_obj.SetOriginalDataObject, *my_args)
        return ret

    def update_cache(self, *args):
        """
        update_cache(self, newObject:DataObject) -> None
        C++: void update_cache(DataObject *newObject)
        Set given dataset as the new Cache. Also update stored MTime from
        original_data_set mesh and Consumer.
        """
        my_args = [deref_vtk(x) for x in args]
        ret = self._wrap_call(self._vtk_obj.UpdateCache, *my_args)
        return ret

    _updateable_traits_ = \
    (('debug', 'GetDebug'), ('global_warning_display',
    'GetGlobalWarningDisplay'), ('object_name', 'GetObjectName'),
    ('reference_count', 'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['debug', 'global_warning_display', 'object_name'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(DataObjectMeshCache, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit DataObjectMeshCache properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View(([], [], ['object_name']),
            title='Edit DataObjectMeshCache properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit DataObjectMeshCache properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

