# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.glsl_modifier_base import GLSLModifierBase


class GLSLModPixelDebugger(GLSLModifierBase):
    r"""
    GLSLModPixelDebugger - Allow live pixel debugging by overwriting
    gl_frag_data[0] output.
    
    Superclass: GLSLModifierBase
    
    This modification significantly simplifies the lives of VTK open_gl
    developers, enabling them to debug and adjust GLSL code in real-time
    while the application is running.
    
    This eliminates the need to recompile VTK for minor shader code
    adjustments. Developers can conveniently keep the JSON and associated
    GLSL files open in a code editor, making changes while a unit test or
    VTK application is actively running. Ultimately, you can just move
    your mouse in the render window to witness your modifications taking
    effect!
    
    Shader substitutions will need to be defined in a json file. An
    example is provided in
    Rendering/cell_grid/live_glsl_debug_sample/sample.json file. If you've
    built VTK from source, you may live edit that json file and glsl
    files under the live_glsl_debug_sample directory. Here is what it looks
    like: {
      "Substitutions": [
        {
          "Target": "//VTK::Light::Impl",
          "shader_type": "Fragment",
          "file_name": "normal-debug.glsl",
          "replace_all_occurrences": false,
          "file_name_is_absolute": false,
          "Enabled": false
        },
        {
          "Target": "//VTK::Light::Impl",
          "shader_type": "Fragment",
          "file_name": "parametric-debug.glsl",
          "replace_all_occurrences": false,
          "file_name_is_absolute": false,
          "Enabled": false
        }
      ] } In the sample, both substitutions are disabled. Please enable
    either to view it. Here is detailed information about the keys:
    
    \li Substitutions: This is a list of maps that contain information
        about a substitution.
    
    \li Target: This must be a string of type
        "//VTK::Feature::[Dec,Impl]".
    These are found in the shader templates.
    
    \li shader_type: This must be either "Fragment" or "Vertex" or
        "Geometry".
    
    \li file_name: Path to a file which has glsl code which will be pasted
    in place of the 'Target'
    string.
    
    \li replace_all_occurrences: Whether to replace all occurrences of
        'Target' string with the
    contents from the 'file_name' file.
    
    \li file_name_is_absolute: Whether 'file_name' is an absolute path or
        relative to the json file.
    
    \li Enabled: When enabled is true, the mod will perform the
        substitution, otherwise, the
    substitution is not applied.
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkGLSLModPixelDebugger, obj, update, **traits)
    
    def set_substitution_json_file_name(self, *args):
        """
        set_substitution_json_file_name(self, filename:str) -> None
        C++: void set_substitution_json_file_name(const std::string &filename)
        Set this to a json file on your file system. Look inside
        Rendering/cell_grid/live_glsl_debug_sample/ for an example.
        """
        ret = self._wrap_call(self._vtk_obj.SetSubstitutionJSONFileName, *args)
        return ret

    _updateable_traits_ = \
    (('debug', 'GetDebug'), ('global_warning_display',
    'GetGlobalWarningDisplay'), ('object_name', 'GetObjectName'),
    ('reference_count', 'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['debug', 'global_warning_display', 'object_name'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(GLSLModPixelDebugger, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit GLSLModPixelDebugger properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View(([], [], ['object_name']),
            title='Edit GLSLModPixelDebugger properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit GLSLModPixelDebugger properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

