# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.writer import Writer


class HDFWriter(Writer):
    r"""
    HDFWriter - Write a PolyData into VTKHDF file.
    
    Superclass: Writer
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkHDFWriter, obj, update, **traits)
    
    chunk_size = traits.Int(25000, enter_set=True, auto_set=False, desc=\
        r"""
        Get/set the chunk size used for chunk storage layout. Chunked
        storage is required for extensible/unlimited dimensions datasets
        (such as time-dependent data), and filters such as compression.
        Read more about chunks and chunk size here :
        https://support.hdfgroup.org/HDF5/doc/Advanced/Chunking/
        
        Regarding performance impact of chunking and how to find the
        optimal value depending on the data, please check this
        documentation:
        https://docs.hdfgroup.org/hdf5/develop/_l_b_dset_layout.html
        
        Defaults to 25000 (to fit with the default chunk cache of 1Mb of
        HDF5).
        """
    )

    def _chunk_size_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetChunkSize,
                        self.chunk_size)

    compression_level = traits.Trait(0, traits.Range(0, 9, enter_set=True, auto_set=False), desc=\
        r"""
        Get/set the compression level used by hdf5. The compression level
        is between 0 (no compression) and 9 (max compression level).
        
        @warning Compression level used can have a big performance impact
        for writing/reading data.
        For reference, the default value used by HDF5 when we apply a
        compression is 4.
        
        ote Only points, cells and data arrays will be compressed. Other
        datas are considered to be too small to be worth compressing.
        
        Default to 0.
        """
    )

    def _compression_level_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetCompressionLevel,
                        self.compression_level)

    def _get_controller(self):
        return wrap_vtk(self._vtk_obj.GetController())
    def _set_controller(self, arg):
        old_val = self._get_controller()
        self._wrap_call(self._vtk_obj.SetController,
                        deref_vtk(arg))
        self.trait_property_changed('controller', old_val, arg)
    controller = traits.Property(_get_controller, _set_controller, desc=\
        r"""
        
        """
    )

    file_name = tvtk_base.vtk_file_name("", desc=\
        r"""
        Get/Set the file name of the HDF file.
        """
    )

    def _file_name_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetFileName,
                        self.file_name)

    overwrite = traits.Bool(True, enter_set=True, auto_set=False, desc=\
        r"""
        Get/set the flag to Overwrite the file if true or fail when the
        file already exists if false. Default is true.
        """
    )

    def _overwrite_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetOverwrite,
                        self.overwrite)

    use_external_composite = traits.Bool(False, enter_set=True, auto_set=False, desc=\
        r"""
        When set, write composite leaf blocks in different files, named
        file_stem__block_name.extension. If file_name does not have an
        extension, blocks are named file_name__block_name.vtkhdf These files
        are referenced by the main file using external links. Default is
        false.
        """
    )

    def _use_external_composite_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetUseExternalComposite,
                        self.use_external_composite)

    use_external_partitions = traits.Bool(False, enter_set=True, auto_set=False, desc=\
        r"""
        When set, write each partition of the input PartitionedDataSet
        in a different file, named file_stem_part_x.extension, where X is
        the partition index. If file_name does not have an extension,
        files are named file_name_part_x.vtkhdf These individual time files
        are referenced by the main file using the HDF5 virtual dataset
        feature, just like the `use_external_time_steps` does. When applied
        to composite datasets, this option forces use_external_composite
        ON. Default is false.
        """
    )

    def _use_external_partitions_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetUseExternalPartitions,
                        self.use_external_partitions)

    use_external_time_steps = traits.Bool(False, enter_set=True, auto_set=False, desc=\
        r"""
        When set, write each time step in a different file. These
        individual time files are referenced by the main file using the
        HDF5 virtual dataset feature. This way, individual time step
        files can be opened by the reader as a non time-dependent
        dataset, and the main file referencing those as a time-dependent
        file seamlessly.
        
        Subfiles are named file_stem__x.extension, where X is the time step
        index. extension defaults to .vtkhdf in case the base filename
        does not have one already.
        
        Note: this option does not support static meshes. Points and
        cells with be copied across time step files. Default is false.
        """
    )

    def _use_external_time_steps_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetUseExternalTimeSteps,
                        self.use_external_time_steps)

    write_all_time_steps = traits.Bool(True, enter_set=True, auto_set=False, desc=\
        r"""
        Get/set the flag to write all timesteps from the input dataset.
        When turned OFF, only write the current timestep.
        """
    )

    def _write_all_time_steps_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetWriteAllTimeSteps,
                        self.write_all_time_steps)

    def _get_input(self):
        try:
            return wrap_vtk(self._vtk_obj.GetInput(0))
        except TypeError:
            return wrap_vtk(self._vtk_obj.GetInput())
    input = traits.Property(_get_input,
                            desc="The first input of this object, i.e. the result of `get_input(0)`.")
    
    def get_input(self, *args):
        """
        get_input(self) -> DataObject
        C++: DataObject *get_input()
        get_input(self, port:int) -> DataObject
        C++: DataObject *get_input(int port)"""
        ret = self._wrap_call(self._vtk_obj.GetInput, *args)
        return wrap_vtk(ret)

    _updateable_traits_ = \
    (('abort_execute', 'GetAbortExecute'), ('release_data_flag',
    'GetReleaseDataFlag'), ('debug', 'GetDebug'),
    ('global_warning_display', 'GetGlobalWarningDisplay'), ('chunk_size',
    'GetChunkSize'), ('compression_level', 'GetCompressionLevel'),
    ('file_name', 'GetFileName'), ('overwrite', 'GetOverwrite'),
    ('use_external_composite', 'GetUseExternalComposite'),
    ('use_external_partitions', 'GetUseExternalPartitions'),
    ('use_external_time_steps', 'GetUseExternalTimeSteps'),
    ('write_all_time_steps', 'GetWriteAllTimeSteps'), ('abort_output',
    'GetAbortOutput'), ('progress_text', 'GetProgressText'),
    ('object_name', 'GetObjectName'), ('reference_count',
    'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['abort_execute', 'debug', 'global_warning_display',
    'release_data_flag', 'abort_output', 'chunk_size',
    'compression_level', 'file_name', 'object_name', 'overwrite',
    'progress_text', 'use_external_composite', 'use_external_partitions',
    'use_external_time_steps', 'write_all_time_steps'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(HDFWriter, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit HDFWriter properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View(([], [], ['abort_output', 'chunk_size', 'compression_level',
            'file_name', 'object_name', 'overwrite', 'use_external_composite',
            'use_external_partitions', 'use_external_time_steps',
            'write_all_time_steps']),
            title='Edit HDFWriter properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit HDFWriter properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

