# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.hyper_tree_grid_algorithm import HyperTreeGridAlgorithm


class HyperTreeGridPlaneCutter(HyperTreeGridAlgorithm):
    r"""
    HyperTreeGridPlaneCutter - cut an hyper tree grid volume with a
    plane and generate a polygonal cut surface.
    
    Superclass: HyperTreeGridAlgorithm
    
    HyperTreeGridPlaneCutter is a filter that takes as input an hyper
    tree grid and a single plane and generates the polygonal data
    intersection surface. This cut is computed at the leaf cells of the
    hyper tree. It is left as an option to decide whether the cut should
    be computed over the original AMR mesh or over its dual; in the
    latter case, perfect connectivity (i.e., mesh conformity in the FE
    sense) is achieved at the cost of interpolation to the dual of the
    input AMR mesh, and therefore of missing intersection plane pieces
    near the primal boundary.
    
    @sa
    HyperTreeGrid HyperTreeGridAlgorithm
    
    @par Thanks: This class was written by Philippe Pebay on a idea of
    Guenole Harel and Jacques-Bernard Lekien, 2016 This class was
    modified by Rogeli Grima Torres, 2016 This class was modified by
    Jacques-Bernard Lekien, 2018 This work was supported by Commissariat
    a l'Energie Atomique CEA, DAM, DIF, F-91297 Arpajon, France.
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkHyperTreeGridPlaneCutter, obj, update, **traits)
    
    dual = tvtk_base.false_bool_trait(desc=\
        r"""
        Set/Get whether output mesh should be computed on dual grid
        """
    )

    def _dual_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetDual,
                        self.dual_)

    plane = traits.Array(enter_set=True, auto_set=False, shape=(None,), dtype="float", value=(0.0, 0.0, 0.0, 0.0), cols=3, desc=\
        r"""
        Specify the plane with its [a,b,c,d] Cartesian coefficients: a*x
        + b*y + c*z = d
        """
    )

    def _plane_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetPlane,
                        self.plane)

    def _get_axis_alignment(self):
        return self._vtk_obj.GetAxisAlignment()
    axis_alignment = traits.Property(_get_axis_alignment, desc=\
        r"""
        Returns 0 if plane's normal is aligned with X axis, 1 if it is
        aligned with Y axis, 2 if it is aligned with Z axis. Returns -1
        if not aligned with any principal axis.
        """
    )

    def is_plane_orthogonal_to_x_axis(self):
        """
        is_plane_orthogonal_to_x_axis(self) -> bool
        C++: bool is_plane_orthogonal_to_x_axis()
        Returns true if plane's normal is aligned with the corresponding
        axis, false elsewise.
        """
        ret = self._vtk_obj.IsPlaneOrthogonalToXAxis()
        return ret
        

    def is_plane_orthogonal_to_y_axis(self):
        """
        is_plane_orthogonal_to_y_axis(self) -> bool
        C++: bool is_plane_orthogonal_to_y_axis()"""
        ret = self._vtk_obj.IsPlaneOrthogonalToYAxis()
        return ret
        

    def is_plane_orthogonal_to_z_axis(self):
        """
        is_plane_orthogonal_to_z_axis(self) -> bool
        C++: bool is_plane_orthogonal_to_z_axis()"""
        ret = self._vtk_obj.IsPlaneOrthogonalToZAxis()
        return ret
        

    _updateable_traits_ = \
    (('dual', 'GetDual'), ('abort_execute', 'GetAbortExecute'),
    ('release_data_flag', 'GetReleaseDataFlag'), ('debug', 'GetDebug'),
    ('global_warning_display', 'GetGlobalWarningDisplay'), ('plane',
    'GetPlane'), ('abort_output', 'GetAbortOutput'), ('progress_text',
    'GetProgressText'), ('object_name', 'GetObjectName'),
    ('reference_count', 'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['abort_execute', 'debug', 'dual', 'global_warning_display',
    'release_data_flag', 'abort_output', 'object_name', 'plane',
    'progress_text'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(HyperTreeGridPlaneCutter, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit HyperTreeGridPlaneCutter properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View((['dual'], [], ['abort_output', 'object_name', 'plane']),
            title='Edit HyperTreeGridPlaneCutter properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit HyperTreeGridPlaneCutter properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

