# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.threaded_image_algorithm import ThreadedImageAlgorithm


class ImageSSIM(ThreadedImageAlgorithm):
    r"""
    ImageSSIM - This filter implements an algorithm based on SSIM for
    image comparison.
    
    Superclass: ThreadedImageAlgorithm
    
    The SSIM (Structural Similarity Index Measure) is a method for
    measuring the similarity between two images. The SSIM index was
    originally proposed in: "Zhou Wang, Alan C. Bovik, Hamid R. Sheikh,
    Eero P. Simoncelli, 'Image Quality Assessment: From Error Visibility
    to Structural Similarity', IEEE Transactions on Image Processing,
    13(4), 600-612, April 2004."
    
    This filter takes 2 images as inputs. It will compare the arrays
    returned by `get_scalars` in the input point data. The 2 arrays need
    te have the same underlying type, and have the same number of
    components. It outputs a heatmap of SSIM values between -1 and 1,
    measuring the structural similarity between 2 corresponding tiles in
    the images. Input values are weighted by a Gaussian kernel whose
    standard deviation is defined as `patch_radius / 3`.
    
    The range of the input arrays is important to know for
    regularizations purposes. 2 constants `c1` and `c2` are set by the
    filter to avoid divisions by zero without distorting the SSIM
    computation too much. If `L` is the range of an input channel, then
    `c1 = 0.0001 * L * L`, and `c2 = 0.0009 * L * L`, as advised by the
    original paper. The range can be automatically computed, or set by
    the user sungi `set_input_to_lab`, `set_input_to_rgb`,
    `set_input_to_grayscale` or `set_input_range`.
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkImageSSIM, obj, update, **traits)
    
    clamp_negative_values = tvtk_base.false_bool_trait(desc=\
        r"""
        When turned ON, negative outputs of the SSIM computation are
        clamped to zero. It is turned OFF by default.
        """
    )

    def _clamp_negative_values_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetClampNegativeValues,
                        self.clamp_negative_values_)

    patch_radius = traits.Float(6.0, enter_set=True, auto_set=False, desc=\
        r"""
        This sets the size of the tile used to compute the SSIM on each
        patch. A pixel x are within the patch centered around a pixel p
        if ||x - p|| <= Radius. It is set to 3 by default.
        """
    )

    def _patch_radius_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetPatchRadius,
                        self.patch_radius)

    def compute_error_metrics(self, *args):
        """
        compute_error_metrics(scalars:DoubleArray, tight:float,
            loose:float) -> None
        C++: static void compute_error_metrics(DoubleArray *scalars,
            double &tight, double &loose)
        Compute error metrics of a provided scalars. Error is defined as
        the maximum of all individual values within the used method.
        Errors are computed using Minkownski and Wasserstein distances.
        Method used are euclidian (tight) or manhattan / earth's mover
        (loose)
        """
        my_args = deref_array(args, [('vtkDoubleArray', 'float', 'float')])
        ret = self._wrap_call(self._vtk_obj.ComputeErrorMetrics, *my_args)
        return ret

    def set_image_connection(self, *args):
        """
        set_image_connection(self, output:AlgorithmOutput) -> None
        C++: void set_image_connection(AlgorithmOutput *output)
        Specify the Image to compare the input to.
        """
        my_args = [deref_vtk(x) for x in args]
        ret = self._wrap_call(self._vtk_obj.SetImageConnection, *my_args)
        return ret

    def set_image_data(self, *args):
        """
        set_image_data(self, image:DataObject) -> None
        C++: void set_image_data(DataObject *image)"""
        my_args = [deref_vtk(x) for x in args]
        ret = self._wrap_call(self._vtk_obj.SetImageData, *my_args)
        return ret

    def set_input_range(self, *args):
        """
        set_input_range(self, range:[int, ...]) -> None
        C++: void set_input_range(std::vector<int> &range)
        Setup the range of each components of the input scalars. If the
        range has not been set, or if the number of components in the
        input does not match the number of provided ranges, it will be
        automatically be set using the input array. The range is used to
        set the constants c1 and c2.
        """
        ret = self._wrap_call(self._vtk_obj.SetInputRange, *args)
        return ret

    def set_input_to_auto(self):
        """
        set_input_to_auto(self) -> None
        C++: void set_input_to_auto()
        The c1 and c2 constant will be computed automatically based on
        the range of each individual components Please note the resulting
        SSIM can be NaN in specific cases.
        """
        ret = self._vtk_obj.SetInputToAuto()
        return ret
        

    def set_input_to_grayscale(self):
        """
        set_input_to_grayscale(self) -> None
        C++: void set_input_to_grayscale()
        Assume the input is in grayscale, using integers from 0 to 255.
        This will set appropriate constants c1 and c2
        """
        ret = self._vtk_obj.SetInputToGrayscale()
        return ret
        

    def set_input_to_lab(self):
        """
        set_input_to_lab(self) -> None
        C++: void set_input_to_lab()
        Assume the input is in Lab format. This will set appropriate
        constants c1 and c2 for each input channel
        """
        ret = self._vtk_obj.SetInputToLab()
        return ret
        

    def set_input_to_rgb(self):
        """
        set_input_to_rgb(self) -> None
        C++: void set_input_to_rgb()
        Assume the input is in RGB format, using integers from 0 to 255.
        This will set appropriate constants c1 and c2 for each input
        channel
        """
        ret = self._vtk_obj.SetInputToRGB()
        return ret
        

    def set_input_to_rgba(self):
        """
        set_input_to_rgba(self) -> None
        C++: void set_input_to_rgba()
        Assume the input is in RGBA format, using integers from 0 to 255.
        This will set appropriate constants c1 and c2 for each input
        channel
        """
        ret = self._vtk_obj.SetInputToRGBA()
        return ret
        

    _updateable_traits_ = \
    (('clamp_negative_values', 'GetClampNegativeValues'),
    ('abort_execute', 'GetAbortExecute'), ('release_data_flag',
    'GetReleaseDataFlag'), ('debug', 'GetDebug'),
    ('global_warning_display', 'GetGlobalWarningDisplay'), ('split_mode',
    'GetSplitMode'), ('patch_radius', 'GetPatchRadius'),
    ('desired_bytes_per_piece', 'GetDesiredBytesPerPiece'), ('enable_smp',
    'GetEnableSMP'), ('global_default_enable_smp',
    'GetGlobalDefaultEnableSMP'), ('minimum_piece_size',
    'GetMinimumPieceSize'), ('number_of_threads', 'GetNumberOfThreads'),
    ('abort_output', 'GetAbortOutput'), ('progress_text',
    'GetProgressText'), ('object_name', 'GetObjectName'),
    ('reference_count', 'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['abort_execute', 'clamp_negative_values', 'debug',
    'global_warning_display', 'release_data_flag', 'split_mode',
    'abort_output', 'desired_bytes_per_piece', 'enable_smp',
    'global_default_enable_smp', 'minimum_piece_size',
    'number_of_threads', 'object_name', 'patch_radius', 'progress_text'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(ImageSSIM, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit ImageSSIM properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View((['clamp_negative_values'], ['split_mode'], ['abort_output',
            'desired_bytes_per_piece', 'enable_smp', 'global_default_enable_smp',
            'minimum_piece_size', 'number_of_threads', 'object_name',
            'patch_radius']),
            title='Edit ImageSSIM properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit ImageSSIM properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

