# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.writer import Writer


class IOSSWriter(Writer):
    r"""
    IOSSWriter - Writer for IOSS (Sierra IO System)
    
    Superclass: Writer
    
    IOSSWriter is a writer that uses the IOSS (Sierra IO System)
    library to write files. Currently this writer supports the Exodus
    file format. IOSS imposes certain restrictions on the aforementioned
    file format and hence it may not be possible to write every dataset
    as an Exodus file using this writer. This is rare for the Exodus.
    
    This writer generates files using the same naming conventions that
    IOSSReader understands.
    
    @section selecting_blocks_sets_to_write Selecting blocks and sets to
    write
    
    An IOSS file comprises of blocks and sets of various types. These are
    described by the enum `IOSSReader::entity_type`. If the input of
    the writer was initially read by the IOSSReader, then the entity
    types can be automatically deduced. If it was not initially read by
    IOSSReader (or even if it was), the assembly name and selectors
    can be utilized to define which blocks are e.g. element blocks, or
    side sets. `IOSSWriter::set_assembly_name(...)` can be used to set
    the assembly, and `IOSSWriter::add_selector(...)` (or one of its
    convenience variants) can be used to add selectors. If no selectors
    have been defined all partitions will be treated as element blocks.
    
    Typical usage is as follows:
    
    {.cpp}
    Newwriter;
    writer->set_input_data(...)
    writer->set_file_name(...);
    writer->set_assembly_name("Assembly");
    writer->add_element_block_selector("/IOSS/element_blocks");
    writer->Write();
    
    @section selecting_arrays_to_write Selecting arrays to write
    
    Similar arrays (or fields as IOSS refers to them) to read from each
    of the blocks or sets can be specified using the
    `vtkdata_array_selection` instance returned using
    `IOSSWriter::get_field_selection` (or one of its convenience
    variants).
    
    By default all arrays are enabled. To write specific arrays, f
    choose_fields_to_write has to be true, which can change using
    `IOSSWriter::set_choose_fields_to_write(true)`.
    
    Typical usage is as follows:
    
    {.cpp}
    Newwriter;
    writer->set_input_data(...)
    writer->set_file_name(...);
    writer->set_assembly_name("Assembly");
    writer->add_element_block_selector("/IOSS/element_blocks");
    writer->set_choose_fields_to_write(true);
    writer->get_element_block_field_selection()->enable_array("EQPS");
    writer->Write();
    
    @section selection_time_steps Selection time_steps
    
    `IOSSWriter::set_time_step_range(...)` and
    `IOSSWriter::set_time_step_stride(...)` can be used to write a subset
    of time steps using a range and a stride.
    
    @section upcoming_features Upcoming features
    
    The following features are planned to be implemented in the future:
    \li Support CGNS file format.
    \li Support writing global data
    \li Support writing pedigree IDs
    
    @section Appendix Appendix
    * [Sierra IO System](https://sandialabs.github.io/seacas-docs)
    * [Exodus file
      format](https://sandialabs.github.io/seacas-docs/exodus_ii-Addendum.p
      df)
    
    @sa
    IOSSReader, ExodusIIWriter
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkIOSSWriter, obj, update, **traits)
    
    choose_fields_to_write = tvtk_base.false_bool_trait(desc=\
        r"""
        Choose which fields to write. If this is true, then only the
        arrays selected will be written. If this is false, then all
        arrays will be written.
        
        The default is false.
        """
    )

    def _choose_fields_to_write_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetChooseFieldsToWrite,
                        self.choose_fields_to_write_)

    offset_global_ids = tvtk_base.false_bool_trait(desc=\
        r"""
        Exodus wants global ids to start with 1, while VTK generally
        produces global ids starting with 0. Set this to true (default
        false), if the global ids are generated by VTK and hence start
        with 0. When writing to the output file, they will be offset by 1
        to ensure the ids are valid exodus ids.
        """
    )

    def _offset_global_ids_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetOffsetGlobalIds,
                        self.offset_global_ids_)

    preserve_input_entity_groups = tvtk_base.true_bool_trait(desc=\
        r"""
        If input is untransformed IOSS dataset, then the writer can
        preserve entity group classifications, such as element blocks,
        side sets etc. The same is not true if the input has been
        transformed e.g. through a clip filter. Thus flag is used to
        indicate whether the input has valid element classifications.
        """
    )

    def _preserve_input_entity_groups_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetPreserveInputEntityGroups,
                        self.preserve_input_entity_groups_)

    preserve_original_ids = tvtk_base.false_bool_trait(desc=\
        r"""
        If input is transformed, e.g. through clipping, new element
        blocks may be created. This flag can be used to indicate whether
        to preserve the original ids from blocks.
        
        The default is false.
        """
    )

    def _preserve_original_ids_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetPreserveOriginalIds,
                        self.preserve_original_ids_)

    write_qa_and_information_records = tvtk_base.true_bool_trait(desc=\
        r"""
        When set to true (default), the writer will write quality
        assurance and information records.
        
        These records are not copied from the input, but they are
        generated by the writer.
        """
    )

    def _write_qa_and_information_records_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetWriteQAAndInformationRecords,
                        self.write_qa_and_information_records_)

    assembly_name = traits.String('Hierarchy', enter_set=True, auto_set=False, desc=\
        r"""
        Get/Set the active assembly to use. The chosen assembly is used
        in combination with the selectors specified to determine which
        blocks are to be extracted.
        
        The default is set to DataAssemblyUtilities::HierarchyName().
        """
    )

    def _assembly_name_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetAssemblyName,
                        self.assembly_name)

    def _get_controller(self):
        return wrap_vtk(self._vtk_obj.GetController())
    def _set_controller(self, arg):
        old_val = self._get_controller()
        self._wrap_call(self._vtk_obj.SetController,
                        deref_vtk(arg))
        self.trait_property_changed('controller', old_val, arg)
    controller = traits.Property(_get_controller, _set_controller, desc=\
        r"""
        
        """
    )

    displacement_magnitude = traits.Trait(1.0, traits.Range(0.0, 1e+299, enter_set=True, auto_set=False), desc=\
        r"""
        If input dataset has displacements pre-applied, setting the
        displacement magnitude to non-zero ensures that the point
        coordinates in the dataset are correctly transformed using the
        displacement field array, if present.
        
        Defaults to 1.0.
        """
    )

    def _displacement_magnitude_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetDisplacementMagnitude,
                        self.displacement_magnitude)

    def get_edge_block_selector(self, *args):
        """
        get_edge_block_selector(self, index:int) -> str
        C++: const char *get_edge_block_selector(int index)"""
        ret = self._wrap_call(self._vtk_obj.GetEdgeBlockSelector, *args)
        return ret

    def set_edge_block_selector(self, *args):
        """
        set_edge_block_selector(self, selector:str) -> None
        C++: void set_edge_block_selector(const char *selector)"""
        ret = self._wrap_call(self._vtk_obj.SetEdgeBlockSelector, *args)
        return ret

    def get_edge_set_selector(self, *args):
        """
        get_edge_set_selector(self, index:int) -> str
        C++: const char *get_edge_set_selector(int index)"""
        ret = self._wrap_call(self._vtk_obj.GetEdgeSetSelector, *args)
        return ret

    def set_edge_set_selector(self, *args):
        """
        set_edge_set_selector(self, selector:str) -> None
        C++: void set_edge_set_selector(const char *selector)"""
        ret = self._wrap_call(self._vtk_obj.SetEdgeSetSelector, *args)
        return ret

    def get_element_block_selector(self, *args):
        """
        get_element_block_selector(self, index:int) -> str
        C++: const char *get_element_block_selector(int index)"""
        ret = self._wrap_call(self._vtk_obj.GetElementBlockSelector, *args)
        return ret

    def set_element_block_selector(self, *args):
        """
        set_element_block_selector(self, selector:str) -> None
        C++: void set_element_block_selector(const char *selector)"""
        ret = self._wrap_call(self._vtk_obj.SetElementBlockSelector, *args)
        return ret

    def get_element_set_selector(self, *args):
        """
        get_element_set_selector(self, index:int) -> str
        C++: const char *get_element_set_selector(int index)"""
        ret = self._wrap_call(self._vtk_obj.GetElementSetSelector, *args)
        return ret

    def set_element_set_selector(self, *args):
        """
        set_element_set_selector(self, selector:str) -> None
        C++: void set_element_set_selector(const char *selector)"""
        ret = self._wrap_call(self._vtk_obj.SetElementSetSelector, *args)
        return ret

    def get_face_block_selector(self, *args):
        """
        get_face_block_selector(self, index:int) -> str
        C++: const char *get_face_block_selector(int index)"""
        ret = self._wrap_call(self._vtk_obj.GetFaceBlockSelector, *args)
        return ret

    def set_face_block_selector(self, *args):
        """
        set_face_block_selector(self, selector:str) -> None
        C++: void set_face_block_selector(const char *selector)"""
        ret = self._wrap_call(self._vtk_obj.SetFaceBlockSelector, *args)
        return ret

    def get_face_set_selector(self, *args):
        """
        get_face_set_selector(self, index:int) -> str
        C++: const char *get_face_set_selector(int index)"""
        ret = self._wrap_call(self._vtk_obj.GetFaceSetSelector, *args)
        return ret

    def set_face_set_selector(self, *args):
        """
        set_face_set_selector(self, selector:str) -> None
        C++: void set_face_set_selector(const char *selector)"""
        ret = self._wrap_call(self._vtk_obj.SetFaceSetSelector, *args)
        return ret

    file_name = tvtk_base.vtk_file_name("", desc=\
        r"""
        Get/set the filename. When writing in a distributed environment,
        the actual filename written out may be different.
        """
    )

    def _file_name_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetFileName,
                        self.file_name)

    maximum_time_steps_per_file = traits.Int(2147483647, enter_set=True, auto_set=False, desc=\
        r"""
        A debugging variable, set this to non-zero positive number to
        save at most the specified number of timesteps in a single file
        before starting a new one. The writer may start new files (aka
        restarts) automatically if it determines that the mesh has
        changed.
        
        Defaults to 0 i.e. unlimited timesteps per file.
        """
    )

    def _maximum_time_steps_per_file_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetMaximumTimeStepsPerFile,
                        self.maximum_time_steps_per_file)

    def get_node_set_selector(self, *args):
        """
        get_node_set_selector(self, index:int) -> str
        C++: const char *get_node_set_selector(int index)"""
        ret = self._wrap_call(self._vtk_obj.GetNodeSetSelector, *args)
        return ret

    def set_node_set_selector(self, *args):
        """
        set_node_set_selector(self, selector:str) -> None
        C++: void set_node_set_selector(const char *selector)"""
        ret = self._wrap_call(self._vtk_obj.SetNodeSetSelector, *args)
        return ret

    remove_ghosts = traits.Bool(True, enter_set=True, auto_set=False, desc=\
        r"""
        Set/Get whether to write remove ghost cells from the input.
        
        The default is 1.
        """
    )

    def _remove_ghosts_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetRemoveGhosts,
                        self.remove_ghosts)

    def _get_selector(self):
        return self._vtk_obj.GetSelector()
    def _set_selector(self, arg):
        old_val = self._get_selector()
        self._wrap_call(self._vtk_obj.SetSelector,
                        arg)
        self.trait_property_changed('selector', old_val, arg)
    selector = traits.Property(_get_selector, _set_selector, desc=\
        r"""
        
        """
    )

    def get_side_set_selector(self, *args):
        """
        get_side_set_selector(self, index:int) -> str
        C++: const char *get_side_set_selector(int index)"""
        ret = self._wrap_call(self._vtk_obj.GetSideSetSelector, *args)
        return ret

    def set_side_set_selector(self, *args):
        """
        set_side_set_selector(self, selector:str) -> None
        C++: void set_side_set_selector(const char *selector)"""
        ret = self._wrap_call(self._vtk_obj.SetSideSetSelector, *args)
        return ret

    time_step_range = traits.Array(enter_set=True, auto_set=False, shape=(None,), dtype="int64", value=(0, 2147483646), cols=2, desc=\
        r"""
        `time_step_range` and `time_step_stride` can be used to limit which
        timesteps will be written.
        
        If the range is invalid, i.e. `time_step_range[0] >=
        time_step_range[1]`, it's assumed that no time_step_range overrides
        have been specified and both time_step_range and time_step_stride
        will be ignored. When valid, only the chosen subset of files will
        be processed.
        """
    )

    def _time_step_range_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetTimeStepRange,
                        self.time_step_range)

    time_step_stride = traits.Trait(1, traits.Range(1, 2147483647, enter_set=True, auto_set=False), desc=\
        r"""
        
        """
    )

    def _time_step_stride_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetTimeStepStride,
                        self.time_step_stride)

    def _get_edge_block_field_selection(self):
        return wrap_vtk(self._vtk_obj.GetEdgeBlockFieldSelection())
    edge_block_field_selection = traits.Property(_get_edge_block_field_selection, desc=\
        r"""
        Returns the field selection object for the edge block arrays.
        """
    )

    def _get_edge_set_field_selection(self):
        return wrap_vtk(self._vtk_obj.GetEdgeSetFieldSelection())
    edge_set_field_selection = traits.Property(_get_edge_set_field_selection, desc=\
        r"""
        Returns the field selection object for the edge set arrays.
        """
    )

    def _get_element_block_field_selection(self):
        return wrap_vtk(self._vtk_obj.GetElementBlockFieldSelection())
    element_block_field_selection = traits.Property(_get_element_block_field_selection, desc=\
        r"""
        Returns the field selection object for the element block arrays.
        """
    )

    def _get_element_set_field_selection(self):
        return wrap_vtk(self._vtk_obj.GetElementSetFieldSelection())
    element_set_field_selection = traits.Property(_get_element_set_field_selection, desc=\
        r"""
        Returns the field selection object for the element set arrays.
        """
    )

    def _get_face_block_field_selection(self):
        return wrap_vtk(self._vtk_obj.GetFaceBlockFieldSelection())
    face_block_field_selection = traits.Property(_get_face_block_field_selection, desc=\
        r"""
        Returns the field selection object for the face block arrays.
        """
    )

    def _get_face_set_field_selection(self):
        return wrap_vtk(self._vtk_obj.GetFaceSetFieldSelection())
    face_set_field_selection = traits.Property(_get_face_set_field_selection, desc=\
        r"""
        Returns the field selection object for the edge set arrays.
        """
    )

    def get_field_selection(self, *args):
        """
        get_field_selection(self, type:IOSSReader.EntityType)
            -> DataArraySelection
        C++: DataArraySelection *get_field_selection(EntityType type)
        Get the selection object for the given entity type. This can be
        used to select which fields to write.
        """
        ret = self._wrap_call(self._vtk_obj.GetFieldSelection, *args)
        return ret

    def _get_input(self):
        try:
            return wrap_vtk(self._vtk_obj.GetInput(0))
        except TypeError:
            return wrap_vtk(self._vtk_obj.GetInput())
    input = traits.Property(_get_input,
                            desc="The first input of this object, i.e. the result of `get_input(0)`.")
    
    def get_input(self, *args):
        """
        get_input(self) -> DataObject
        C++: DataObject *get_input()
        get_input(self, port:int) -> DataObject
        C++: DataObject *get_input(int port)"""
        ret = self._wrap_call(self._vtk_obj.GetInput, *args)
        return wrap_vtk(ret)

    def _get_node_block_field_selection(self):
        return wrap_vtk(self._vtk_obj.GetNodeBlockFieldSelection())
    node_block_field_selection = traits.Property(_get_node_block_field_selection, desc=\
        r"""
        Returns the field selection object for the element block arrays.
        """
    )

    def _get_node_set_field_selection(self):
        return wrap_vtk(self._vtk_obj.GetNodeSetFieldSelection())
    node_set_field_selection = traits.Property(_get_node_set_field_selection, desc=\
        r"""
        Returns the field selection object for the node set arrays.
        """
    )

    def _get_number_of_edge_block_selectors(self):
        return self._vtk_obj.GetNumberOfEdgeBlockSelectors()
    number_of_edge_block_selectors = traits.Property(_get_number_of_edge_block_selectors, desc=\
        r"""
        
        """
    )

    def _get_number_of_edge_set_selectors(self):
        return self._vtk_obj.GetNumberOfEdgeSetSelectors()
    number_of_edge_set_selectors = traits.Property(_get_number_of_edge_set_selectors, desc=\
        r"""
        
        """
    )

    def _get_number_of_element_block_selectors(self):
        return self._vtk_obj.GetNumberOfElementBlockSelectors()
    number_of_element_block_selectors = traits.Property(_get_number_of_element_block_selectors, desc=\
        r"""
        
        """
    )

    def _get_number_of_element_set_selectors(self):
        return self._vtk_obj.GetNumberOfElementSetSelectors()
    number_of_element_set_selectors = traits.Property(_get_number_of_element_set_selectors, desc=\
        r"""
        
        """
    )

    def _get_number_of_face_block_selectors(self):
        return self._vtk_obj.GetNumberOfFaceBlockSelectors()
    number_of_face_block_selectors = traits.Property(_get_number_of_face_block_selectors, desc=\
        r"""
        
        """
    )

    def _get_number_of_face_set_selectors(self):
        return self._vtk_obj.GetNumberOfFaceSetSelectors()
    number_of_face_set_selectors = traits.Property(_get_number_of_face_set_selectors, desc=\
        r"""
        
        """
    )

    def _get_number_of_node_set_selectors(self):
        return self._vtk_obj.GetNumberOfNodeSetSelectors()
    number_of_node_set_selectors = traits.Property(_get_number_of_node_set_selectors, desc=\
        r"""
        
        """
    )

    def get_number_of_selectors(self, *args):
        """
        get_number_of_selectors(self, entity:IOSSReader.EntityType) -> int
        C++: int get_number_of_selectors(EntityType entity)
        API to access entity selectors.
        """
        ret = self._wrap_call(self._vtk_obj.GetNumberOfSelectors, *args)
        return ret

    def _get_number_of_side_set_selectors(self):
        return self._vtk_obj.GetNumberOfSideSetSelectors()
    number_of_side_set_selectors = traits.Property(_get_number_of_side_set_selectors, desc=\
        r"""
        
        """
    )

    def _get_side_set_field_selection(self):
        return wrap_vtk(self._vtk_obj.GetSideSetFieldSelection())
    side_set_field_selection = traits.Property(_get_side_set_field_selection, desc=\
        r"""
        Returns the field selection object for the side set arrays.
        """
    )

    def add_edge_block_selector(self, *args):
        """
        add_edge_block_selector(self, selector:str) -> bool
        C++: bool add_edge_block_selector(const char *selector)
        Add/Clear/Set/Get edge block selectors
        """
        ret = self._wrap_call(self._vtk_obj.AddEdgeBlockSelector, *args)
        return ret

    def add_edge_set_selector(self, *args):
        """
        add_edge_set_selector(self, selector:str) -> bool
        C++: bool add_edge_set_selector(const char *selector)
        Add/Clear/Set/Get edge set selectors
        """
        ret = self._wrap_call(self._vtk_obj.AddEdgeSetSelector, *args)
        return ret

    def add_element_block_selector(self, *args):
        """
        add_element_block_selector(self, selector:str) -> bool
        C++: bool add_element_block_selector(const char *selector)
        Add/Clear/Set/Get element block selectors
        """
        ret = self._wrap_call(self._vtk_obj.AddElementBlockSelector, *args)
        return ret

    def add_element_set_selector(self, *args):
        """
        add_element_set_selector(self, selector:str) -> bool
        C++: bool add_element_set_selector(const char *selector)
        Add/Clear/Set/Get element set selectors
        """
        ret = self._wrap_call(self._vtk_obj.AddElementSetSelector, *args)
        return ret

    def add_face_block_selector(self, *args):
        """
        add_face_block_selector(self, selector:str) -> bool
        C++: bool add_face_block_selector(const char *selector)
        Add/Clear/Set/Get face block selectors
        """
        ret = self._wrap_call(self._vtk_obj.AddFaceBlockSelector, *args)
        return ret

    def add_face_set_selector(self, *args):
        """
        add_face_set_selector(self, selector:str) -> bool
        C++: bool add_face_set_selector(const char *selector)
        Add/Clear/Set/Get edge set selectors
        """
        ret = self._wrap_call(self._vtk_obj.AddFaceSetSelector, *args)
        return ret

    def add_node_set_selector(self, *args):
        """
        add_node_set_selector(self, selector:str) -> bool
        C++: bool add_node_set_selector(const char *selector)
        Add/Clear/Set/Get node set selectors
        """
        ret = self._wrap_call(self._vtk_obj.AddNodeSetSelector, *args)
        return ret

    def add_selector(self, *args):
        """
        add_selector(self, entity:IOSSReader.EntityType, selector:str)
            -> bool
        C++: bool add_selector(EntityType entity, const char *selector)
        API to set entity selectors. Multiple selectors can be added
        using `add_selector`. The order in which selectors are specified
        is not preserved and has no impact on the result.
        
        `add_selector` returns true if the selector was added, false if
        the selector was already specified and hence not added.
        
        @sa DataAssembly::SelectNodes
        """
        ret = self._wrap_call(self._vtk_obj.AddSelector, *args)
        return ret

    def add_side_set_selector(self, *args):
        """
        add_side_set_selector(self, selector:str) -> bool
        C++: bool add_side_set_selector(const char *selector)
        Add/Clear/Set/Get side set selectors
        """
        ret = self._wrap_call(self._vtk_obj.AddSideSetSelector, *args)
        return ret

    def clear_edge_block_selectors(self):
        """
        clear_edge_block_selectors(self) -> None
        C++: void clear_edge_block_selectors()"""
        ret = self._vtk_obj.ClearEdgeBlockSelectors()
        return ret
        

    def clear_edge_set_selectors(self):
        """
        clear_edge_set_selectors(self) -> None
        C++: void clear_edge_set_selectors()"""
        ret = self._vtk_obj.ClearEdgeSetSelectors()
        return ret
        

    def clear_element_block_selectors(self):
        """
        clear_element_block_selectors(self) -> None
        C++: void clear_element_block_selectors()"""
        ret = self._vtk_obj.ClearElementBlockSelectors()
        return ret
        

    def clear_element_set_selectors(self):
        """
        clear_element_set_selectors(self) -> None
        C++: void clear_element_set_selectors()"""
        ret = self._vtk_obj.ClearElementSetSelectors()
        return ret
        

    def clear_face_block_selectors(self):
        """
        clear_face_block_selectors(self) -> None
        C++: void clear_face_block_selectors()"""
        ret = self._vtk_obj.ClearFaceBlockSelectors()
        return ret
        

    def clear_face_set_selectors(self):
        """
        clear_face_set_selectors(self) -> None
        C++: void clear_face_set_selectors()"""
        ret = self._vtk_obj.ClearFaceSetSelectors()
        return ret
        

    def clear_node_set_selectors(self):
        """
        clear_node_set_selectors(self) -> None
        C++: void clear_node_set_selectors()"""
        ret = self._vtk_obj.ClearNodeSetSelectors()
        return ret
        

    def clear_selectors(self, *args):
        """
        clear_selectors(self, entity:IOSSReader.EntityType) -> None
        C++: void clear_selectors(EntityType entity)"""
        ret = self._wrap_call(self._vtk_obj.ClearSelectors, *args)
        return ret

    def clear_side_set_selectors(self):
        """
        clear_side_set_selectors(self) -> None
        C++: void clear_side_set_selectors()"""
        ret = self._vtk_obj.ClearSideSetSelectors()
        return ret
        

    _updateable_traits_ = \
    (('choose_fields_to_write', 'GetChooseFieldsToWrite'),
    ('offset_global_ids', 'GetOffsetGlobalIds'),
    ('preserve_input_entity_groups', 'GetPreserveInputEntityGroups'),
    ('preserve_original_ids', 'GetPreserveOriginalIds'),
    ('write_qa_and_information_records',
    'GetWriteQAAndInformationRecords'), ('abort_execute',
    'GetAbortExecute'), ('release_data_flag', 'GetReleaseDataFlag'),
    ('debug', 'GetDebug'), ('global_warning_display',
    'GetGlobalWarningDisplay'), ('assembly_name', 'GetAssemblyName'),
    ('displacement_magnitude', 'GetDisplacementMagnitude'), ('file_name',
    'GetFileName'), ('maximum_time_steps_per_file',
    'GetMaximumTimeStepsPerFile'), ('remove_ghosts', 'GetRemoveGhosts'),
    ('time_step_range', 'GetTimeStepRange'), ('time_step_stride',
    'GetTimeStepStride'), ('abort_output', 'GetAbortOutput'),
    ('progress_text', 'GetProgressText'), ('object_name',
    'GetObjectName'), ('reference_count', 'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['abort_execute', 'choose_fields_to_write', 'debug',
    'global_warning_display', 'offset_global_ids',
    'preserve_input_entity_groups', 'preserve_original_ids',
    'release_data_flag', 'write_qa_and_information_records',
    'abort_output', 'assembly_name', 'displacement_magnitude',
    'file_name', 'maximum_time_steps_per_file', 'object_name',
    'progress_text', 'remove_ghosts', 'time_step_range',
    'time_step_stride'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(IOSSWriter, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit IOSSWriter properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View((['choose_fields_to_write', 'offset_global_ids',
            'preserve_input_entity_groups', 'preserve_original_ids',
            'write_qa_and_information_records'], [], ['abort_output',
            'assembly_name', 'displacement_magnitude', 'file_name',
            'maximum_time_steps_per_file', 'object_name', 'remove_ghosts',
            'time_step_range', 'time_step_stride']),
            title='Edit IOSSWriter properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit IOSSWriter properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

