# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.cell3d import Cell3D


class Polyhedron(Cell3D):
    r"""
    Polyhedron - A 3D cell defined by a set of polygonal faces
    
    Superclass: Cell3D
    
    @section Instantiation Instantiation
    
    Polyhedron is a concrete implementation of Cell that represents
    a 3D cell defined by a set of polygonal faces.
    
    To instantiate a Polyhedron, like any Cell, one needs to define
    the following structures:
    - A list of point coordinates
    - A list of global point IDs
    
    Note that the ordering of points coordinates or IDs is not important.
    However, it MUST be consistent between the two lists.
    
    Unlike other kinds of cells (e.g. Voxel), the topology is not
    directly deduced from points coordinates or point IDs ordering; it
    must be explicitly defined by providing a list of faces (see the
    set_faces() method). Each face is represented as a sequence of global
    point Ids.
    
    Once point coordinates, point IDs and faces are defined, the
    Initialize() method should be called in order to setup the internal
    structures and finalize the construction of the polyhedron.
    
    Here is an example of Polyhedron instantiation:{.cpp}
    
    //  9 +------+.11 //    |`.    | `. //    |13`+--+---+ 15 //    |   |
     |   | //  8 +---+--+.10| //     `. |    `.| //     12`+------+ 14 //
    // (Global IDs are arbitrarily chosen between 8 and 15)
    
    // Insert point coordinates
    polyhedron->get_points()->set_number_of_points(8);
    polyhedron->get_points()->set_point(0, 0., 0., 0.); // 8
    polyhedron->get_points()->set_point(1, 0., 1., 0.); // 9
    polyhedron->get_points()->set_point(2, 1., 0., 0.); // 10
    polyhedron->get_points()->set_point(3, 1., 1., 0.); // 11
    polyhedron->get_points()->set_point(4, 0., 0., 1.); // 12
    polyhedron->get_points()->set_point(5, 0., 1., 1.); // 13
    polyhedron->get_points()->set_point(6, 1., 0., 1.); // 14
    polyhedron->get_points()->set_point(7, 1., 1., 1.); // 15
    
    // Insert point IDs (global IDs) // Note that the canonical ordering
    (0, 1, ..., 8) is used // to correlate point Ids and coordinates
    polyhedron->get_point_ids()->Allocate(8); for (int i = 8; i < 16; ++i)
    {
      polyhedron->get_point_ids()->insert_next_id(i); }
    
    // Describe faces, indexed on global IDs IdType faces[31] = { 6,
    // Number of faces
                            4, 9 , 11, 10, 8 , // Number of points in the
    face + point IDs
                            4, 11, 15, 14, 10, // Faces are described
    counter-clockwise
                            4, 15, 13, 12, 14, // looking from the
    "outside" of the cell
                            4, 13, 9 , 8 , 12,
                            4, 14, 12, 8 , 10,
                            4, 13, 15, 11, 9 };
    
    polyhedron->set_faces(faces);
    
    // Initialize the polyhedron // This will build internal structures
    and should be done before the proper // use of the cell.
    polyhedron->Initialize();
    
    @section Specifications Specifications
    
    Polyhedrons described by this class must conform to some criteria in
    order to avoid errors and guarantee good results in terms of
    visualization and processing.
    
    These specifications are described as follows. Polyhedrons must:
    - be watertight : the faces describing the polyhedron should define
      an enclosed volume i.e. define the “inside” and the
      “outside” of the cell
    - have planar faces : all points defining a face should be in the
      same 2D plane
    - not be self-intersecting : for example, a face of the polyhedron
      can’t intersect other ones
    - not contain zero-thickness portions : adjacent faces should not
      overlap each other even partially
    - not contain disconnected elements : detached vertice(s), edge(s) or
    face(s)
    - be simply connected : Polyhedron must describe a single
      polyhedron
    - not contain duplicate elements : each point index and each face
      description should be unique
    - not contain “internal” or “external” faces : for each face,
    one side should be “inside” the cell, the other side
      “outside”
    
    In a more global perspective, polyhedrons must be watertight and
    manifold. In particular, each edge of the polyhedron must be adjacent
    to exactly two faces. Several algorithms like contour, clip or slice
    will assume that each edge of the polyhedron is adjacent to exactly
    two faces and will definitely lead to bad results (and generate
    numerous warnings) if this criterion is not fulfilled.
    
    @section Limitations Limitations
    
    The class does not require the polyhedron to be convex. However, the
    support of concave polyhedrons is currently limited. Concavity can
    lead to bad results with some filters, including:
    - Contour: the contour (surface) can be constructed outside of the
      cell,
    - Triangulate: the current tetrahedralization algorithm can modify
      the initial shape of the polygon (created tetrahedrons can change
      concave portions of the shape to convex ones).
    
    @section other_details Other details
    
    Interpolation functions and weights are defined / computed using the
    method of Mean Value Coordinates (MVC). See the VTK class
    MeanValueCoordinatesInterpolator for more information.
    
    @sa
    Cell3D ConvexPointSet MeanValueCoordinatesInterpolator
    PolyhedronUtilities
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkPolyhedron, obj, update, **traits)
    
    def _get_cell_faces(self):
        return wrap_vtk(self._vtk_obj.GetCellFaces())
    def _set_cell_faces(self, arg):
        old_val = self._get_cell_faces()
        my_arg = deref_array([arg], [['vtkCellArray']])
        self._wrap_call(self._vtk_obj.SetCellFaces,
                        my_arg[0])
        self.trait_property_changed('cell_faces', old_val, arg)
    cell_faces = traits.Property(_get_cell_faces, _set_cell_faces, desc=\
        r"""
        Get the faces of the polyhedron. Face are expressed as sequences
        of global point IDs .
        
        @param faces CellArray that stores the list of polygonal faces
        with their corresponding
        global point IDs
        """
    )

    def _get_faces(self):
        return self._vtk_obj.GetFaces()
    def _set_faces(self, arg):
        old_val = self._get_faces()
        self._wrap_call(self._vtk_obj.SetFaces,
                        arg)
        self.trait_property_changed('faces', old_val, arg)
    faces = traits.Property(_get_faces, _set_faces, desc=\
        r"""
        Get the faces of the polyhedron. Face are expressed as sequences
        of global point IDs .
        
        @return A 1-dimentional array with the following structure :
        ``` [ nb_of_pts_face1, face1Pt1, face1Pt2, …, face1pt_nb_of_pts_face1,
          nb_of_pts_face2, face2Pt1, face2Pt2, …, face2pt_nb_of_pts_face2,
          …,
          nb_of_pts_face_n, faceNPt1, faceNPt2, …, faceNpt_nb_of_pts_face_n ]
        ``` This ordering corresponds to the legacy CellArray form.
        Note that unlike the set_faces method, the total faces number
        leading count is missing. In order to get the number of faces,
        please use the Polyhedron::GetNumberOfFaces() method.
        """
    )

    def _get_poly_data(self):
        return wrap_vtk(self._vtk_obj.GetPolyData())
    poly_data = traits.Property(_get_poly_data, desc=\
        r"""
        Construct polydata if no one exist, then return this->poly_data
        """
    )

    def is_convex(self):
        """
        is_convex(self) -> bool
        C++: bool is_convex()
        Determine whether or not a polyhedron is convex. This method is
        adapted from Devillers et al., "Checking the Convexity of
        Polytopes and the Planarity of Subdivisions", Computational
        Geometry, Volume 11, Issues 3 - 4, December 1998, Pages 187 -
        208.
        """
        ret = self._vtk_obj.IsConvex()
        return ret
        

    def is_inside(self, *args):
        """
        is_inside(self, x:(float, float, float), tolerance:float) -> int
        C++: int is_inside(const double x[3], double tolerance)
        A method particular to Polyhedron. It determines whether a
        point x[3] is inside the polyhedron or not (returns 1 is the
        point is inside, 0 otherwise). The tolerance is expressed in
        normalized space; i.e., a fraction of the size of the bounding
        box.
        """
        ret = self._wrap_call(self._vtk_obj.IsInside, *args)
        return ret

    def triangulate_faces(self, *args):
        """
        triangulate_faces(self, newFaces:IdList) -> int
        C++: int triangulate_faces(IdList *newFaces)
        triangulate_faces(self, newFaces:CellArray) -> int
        C++: int triangulate_faces(CellArray *newFaces)
        Triangulate each face of the polyhedron. This method internally
        use the Cell::Triangulate method on each face (so the
        triangulation method vary depending on the 2D cell type
        corresponding to the face).
        @warning Can lead to bad results with non-planar faces.
        """
        my_args = deref_array(args, [['vtkIdList'], ['vtkCellArray']])
        ret = self._wrap_call(self._vtk_obj.TriangulateFaces, *my_args)
        return ret

    _updateable_traits_ = \
    (('debug', 'GetDebug'), ('global_warning_display',
    'GetGlobalWarningDisplay'), ('merge_tolerance', 'GetMergeTolerance'),
    ('object_name', 'GetObjectName'), ('reference_count',
    'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['debug', 'global_warning_display', 'merge_tolerance',
    'object_name'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(Polyhedron, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit Polyhedron properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View(([], [], ['merge_tolerance', 'object_name']),
            title='Edit Polyhedron properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit Polyhedron properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

